@@ -10,8 +10,10 @@
 NULL
 
 the <- new_environment()
-the$throttle <- list()
+the$throttle <- new_environment()
+the$breaker <- new_environment()
 the$cache_throttle <- list()
 the$token_cache <- new_environment()
 the$last_response <- NULL
 the$last_request <- NULL
+the$pool_pollers <- new_environment()
