@@ -20,33 +20,27 @@
 #' @export
 req_oauth <- function(req, flow, flow_params, cache) {
   # Want req object to contain meaningful objects, not just a closure
-  req_policies(req,
-    auth_oauth = list(
-      cache = cache,
-      flow = flow,
-      flow_params = flow_params
-    )
+  req <- req_auth_sign(req,
+    fun = auth_oauth_sign,
+    params = list(flow = flow, flow_params = flow_params),
+    cache = cache
   )
+  req <- req_policies(req, auth_oauth = TRUE)
+  req
 }
 
-auth_oauth_sign <- function(req, reauth = FALSE) {
-  if (!req_policy_exists(req, "auth_oauth")) {
-    return(req)
-  }
-
+auth_oauth_sign <- function(req, cache, flow, flow_params) {
   token <- auth_oauth_token_get(
-    cache = req$policies$auth_oauth$cache,
-    flow = req$policies$auth_oauth$flow,
-    flow_params = req$policies$auth_oauth$flow_params,
-    reauth = reauth
+    cache = cache,
+    flow = flow,
+    flow_params = flow_params
   )
-
   req_auth_bearer_token(req, token$access_token)
 }
 
-auth_oauth_token_get <- function(cache, flow, flow_params = list(), reauth = FALSE) {
+auth_oauth_token_get <- function(cache, flow, flow_params = list()) {
   token <- cache$get()
-  if (reauth || is.null(token)) {
+  if (is.null(token)) {
     token <- exec(flow, !!!flow_params)
     cache$set(token)
   } else if (token_has_expired(token)) {
@@ -102,13 +96,15 @@ oauth_token_cached <- function(client,
                                reauth = FALSE) {
   check_bool(reauth)
   cache <- cache_choose(client, cache_disk, cache_key)
+  if (reauth) {
+    cache$clear()
+  }
 
   flow_params$client <- client
   auth_oauth_token_get(
     cache = cache,
     flow = flow,
-    flow_params = flow_params,
-    reauth = reauth
+    flow_params = flow_params
   )
 }
 
@@ -117,7 +113,7 @@ resp_is_invalid_oauth_token <- function(req, resp) {
     return(FALSE)
   }
 
-  if (resp_status(resp) != 401) {
+  if (is_error(resp) || resp_status(resp) != 401) {
     return(FALSE)
   }
 
@@ -193,3 +189,16 @@ oauth_cache_path <- function() {
 
   rappdirs::user_cache_dir("httr2")
 }
+
+
+#' Clear OAuth cache
+#'
+#' Use this function to clear cached credentials.
+#'
+#' @export
+#' @inheritParams req_oauth_auth_code
+oauth_cache_clear <- function(client, cache_disk = FALSE, cache_key = NULL) {
+  cache <- cache_choose(client, cache_disk, cache_key)
+  cache$clear()
+  invisible()
+}
