@@ -5,8 +5,8 @@
 #'
 #' @inheritParams req_perform
 #' @param username User name.
-#' @param password Password. You avoid entering the password directly when
-#'   calling this function as it will be captured by `.Rhistory`. Instead,
+#' @param password Password. You should avoid entering the password directly
+#'   when calling this function as it will be captured by `.Rhistory`. Instead,
 #'   leave it unset and the default behaviour will prompt you for it
 #'   interactively.
 #' @returns A modified HTTP [request].
@@ -30,7 +30,7 @@ req_auth_basic <- function(req, username, password = NULL) {
   password <- check_password(password)
 
   username_password <- openssl::base64_encode(paste0(username, ":", password))
-  req_headers(req, Authorization = paste0("Basic ", username_password), .redact = "Authorization")
+  req_headers(req, Authorization = paste0("Basic ", username_password))
 }
 
 #' Authenticate request with bearer token
@@ -57,5 +57,5 @@ req_auth_basic <- function(req, username, password = NULL) {
 req_auth_bearer_token <- function(req, token) {
   check_request(req)
   check_string(token)
-  req_headers(req, Authorization = paste("Bearer", token), .redact = "Authorization")
+  req_headers(req, Authorization = paste("Bearer", token))
 }
