@@ -11,9 +11,7 @@
 #' Adding a body to a request will automatically switch the method to POST.
 #'
 #' @inheritParams req_perform
-#' @param type MIME content type. You shouldn't generally need to specify this as
-#'  the defaults are usually pretty good, e.g. `req_body_file()` will guess it
-#'  from the extension of of `path`. Will be ignored if you have manually set
+#' @param type MIME content type. Will be ignored if you have manually set
 #'  a `Content-Type` header.
 #' @returns A modified HTTP [request].
 #' @examples
@@ -139,7 +137,8 @@ req_body_json_modify <- function(req, ...) {
 #'   data in the body.
 #'
 #'   * For `req_body_form()`, the values must be strings (or things easily
-#'     coerced to strings);
+#'     coerced to strings). Vectors are convertd to strings using the
+#'     value of `.multi`.
 #'   * For `req_body_multipart()` the values must be strings or objects
 #'     produced by [curl::form_file()]/[curl::form_data()].
 #'   * For `req_body_json_modify()`, any simple data made from atomic vectors
@@ -219,6 +218,22 @@ req_body_info <- function(req) {
   }
 }
 
+req_body_get <- function(req) {
+  if (is.null(req$body)) {
+    return("")
+  }
+  switch(
+    req$body$type,
+    raw = req$body$data,
+    form = {
+      data <- unobfuscate(req$body$data)
+      url_query_build(data)
+    },
+    json = exec(jsonlite::toJSON, req$body$data, !!!req$body$params),
+    cli::cli_abort("Unsupported request body type {.str {req$body$type}}.")
+  )
+}
+
 req_body_apply <- function(req) {
   if (is.null(req$body)) {
     return(req)
@@ -245,14 +260,12 @@ req_body_apply <- function(req) {
   } else if (type == "raw") {
     req <- req_body_apply_raw(req, data)
   } else if (type == "json") {
-    json <- exec(jsonlite::toJSON, data, !!!req$body$params)
-    req <- req_body_apply_raw(req, json)
+    req <- req_body_apply_raw(req, req_body_get(req))
   } else if (type == "multipart") {
     data <- unobfuscate(data)
     req$fields <- data
   } else if (type == "form") {
-    data <- unobfuscate(data)
-    req <- req_body_apply_raw(req, query_build(data))
+    req <- req_body_apply_raw(req, req_body_get(req))
   } else {
     cli::cli_abort("Unsupported request body {.arg type}.", .internal = TRUE)
   }
