@@ -15,7 +15,12 @@
 #' [HTTP caching](https://developer.mozilla.org/en-US/docs/Web/HTTP/Caching).
 #'
 #' @inheritParams req_perform
-#' @param path Path to cache directory.
+#' @param path Path to cache directory. Will be created automatically if it
+#'   does not exist.
+#'
+#'   For quick and easy caching within a session, you can use `tempfile()`.
+#'   To cache requests within a package, you can use something like
+#'   `file.path(tools::R_user_dir("pkgdown", "cache"), "httr2")`.
 #'
 #'   httr2 doesn't provide helpers to manage the cache, but if you want to
 #'   empty it, you can use something like
@@ -54,7 +59,7 @@
 req_cache <- function(req,
                       path,
                       use_on_error = FALSE,
-                      debug = FALSE,
+                      debug = getOption("httr2_cache_debug", FALSE),
                       max_age = Inf,
                       max_n = Inf,
                       max_size = 1024^3) {
@@ -87,24 +92,36 @@ cache_debug <- function(req) {
 
 # Cache management --------------------------------------------------------
 
-cache_exists <- function(req) {
-  if (!req_policy_exists(req, "cache_path")) {
-    FALSE
-  } else {
-    file.exists(req_cache_path(req))
-  }
+cache_active <- function(req) {
+  req_policy_exists(req, "cache_path")
 }
 
-# Callers responsibility to check that cache exists
 cache_get <- function(req) {
-  path <- req_cache_path(req)
+  # This check should be redudant but we keep it in for safety
+  if (!cache_active(req)) {
+    return(req)
+  }
 
-  touch(path)
-  readRDS(path)
+  path <- req_cache_path(req)
+  if (!file.exists(path)) {
+    return(NULL)
+  }
+
+  tryCatch(
+    {
+      rds <- readRDS(path)
+      # Update file time if read successfully
+      Sys.setFileTime(path, Sys.time())
+      rds
+    },
+    error = function(e) NULL
+  )
 }
 
 cache_set <- function(req, resp) {
-  if (is_path(resp$body)) {
+  signal("", "httr2_cache_save")
+
+  if (resp_body_type(resp) == "disk") {
     body_path <- req_cache_path(req, ".body")
     file.copy(resp$body, body_path, overwrite = TRUE)
     resp$body <- new_path(body_path)
@@ -164,21 +181,34 @@ cache_prune_files <- function(info, to_remove, why, debug = TRUE) {
 # Hooks for req_perform -----------------------------------------------------
 
 # Can return request or response
-cache_pre_fetch <- function(req) {
-  if (!cache_exists(req)) {
+cache_pre_fetch <- function(req, path = NULL) {
+  if (!cache_active(req)) {
+    return(req)
+  }
+
+  # Only GET requests should be retrieved from cache. It's not sufficient to
+  # only save GET requests, because the method is not part of the cache key
+  if (req_method_get(req) != "GET") {
     return(req)
   }
 
   debug <- cache_debug(req)
   cache_prune_if_needed(req, debug = debug)
 
-  info <- resp_cache_info(cache_get(req))
+  cached_resp <- cache_get(req)
+  if (is.null(cached_resp)) {
+    return(req)
+  }
   if (debug) cli::cli_text("Found url in cache {.val {hash(req$url)}}")
 
+  info <- resp_cache_info(cached_resp)
   if (!is.na(info$expires) && info$expires >= Sys.time()) {
     signal("", "httr2_cache_cached")
-    if (debug) cli::cli_text("Cached value is fresh; retrieving response from cache")
-    cache_get(req)
+    if (debug) cli::cli_text("Cached value is fresh; using response from cache")
+
+    resp <- cached_resp
+    resp$body <- cache_body(cached_resp, path)
+    resp
   } else {
     if (debug) cli::cli_text("Cached value is stale; checking for updates")
     req_headers(req,
@@ -188,32 +218,37 @@ cache_pre_fetch <- function(req) {
   }
 }
 
+# Always returns response
 cache_post_fetch <- function(req, resp, path = NULL) {
-  if (!req_policy_exists(req, "cache_path")) {
+  if (!cache_active(req)) {
     return(resp)
   }
+
   debug <- cache_debug(req)
+  cached_resp <- cache_get(req)
 
   if (is_error(resp)) {
-    if (cache_use_on_error(req) && cache_exists(req)) {
+    if (cache_use_on_error(req) && !is.null(cached_resp)) {
       if (debug) cli::cli_text("Request errored; retrieving response from cache")
-      cache_get(req)
+      cached_resp
     } else {
       resp
     }
-  } else if (resp_status(resp) == 304 && cache_exists(req)) {
+  } else if (resp_status(resp) == 304 && !is.null(cached_resp)) {
     signal("", "httr2_cache_not_modified")
     if (debug) cli::cli_text("Cached value still ok; retrieving body from cache")
 
-    # Replace body with cached result
-    resp$body <- cache_body(req, path)
-
     # Combine headers
-    resp$headers <- cache_headers(req, resp)
+    resp$headers <- cache_headers(cached_resp, resp)
+    # Replace body with cached result
+    resp$body <- cache_body(cached_resp, path)
 
+    # Re-cache, so we get any new headers
+    cache_set(req, resp)
     resp
   } else if (resp_is_cacheable(resp)) {
     if (debug) cli::cli_text("Saving response to cache {.val {hash(req$url)}}")
+
     cache_set(req, resp)
     resp
   } else {
@@ -221,25 +256,29 @@ cache_post_fetch <- function(req, resp, path = NULL) {
   }
 }
 
-cache_body <- function(req, path = NULL) {
-  body <- cache_get(req)$body
+cache_body <- function(cached_resp, path = NULL) {
+  check_response(cached_resp)
 
+  body <- cached_resp$body
   if (is.null(path)) {
     return(body)
   }
 
-  if (is_path(body)) {
-    file.copy(body, path, overwrite = TRUE)
-  } else {
-    writeBin(body, path)
-  }
+  switch(resp_body_type(cached_resp),
+    disk = file.copy(body, path, overwrite = TRUE),
+    memory = writeBin(body, path),
+    stream = cli::cli_abort("Invalid body type", .internal = TRUE)
+  )
+
   new_path(path)
 }
 
-cache_headers <- function(req, resp) {
-  # https://www.rfc-editor.org/rfc/rfc7232#section-4.1
-  cached_headers <- cache_get(req)$headers
-  as_headers(modify_list(cached_headers, !!!resp$headers))
+# https://www.rfc-editor.org/rfc/rfc7232#section-4.1
+cache_headers <- function(cached_resp, resp) {
+  check_response(cached_resp)
+
+  headers <- modify_list(cached_resp$headers, !!!resp$headers, .ignore_case = TRUE)
+  as_headers(headers)
 }
 
 # Caching headers ---------------------------------------------------------
@@ -253,6 +292,10 @@ resp_is_cacheable <- function(resp, control = NULL) {
     return(FALSE)
   }
 
+  if (resp_body_type(resp) == "stream") {
+    return(FALSE)
+  }
+
   control <- control %||% resp_cache_control(resp)
   if ("no-store" %in% control$flags) {
     return(FALSE)
@@ -296,11 +339,11 @@ resp_cache_control <- function(resp) {
     return(NULL)
   }
 
-  pieces <- strsplit(x, ",")[[1]]
+  pieces <- strsplit(x, ",", fixed = TRUE)[[1]]
   pieces <- gsub("^\\s+|\\s+$", "", pieces)
   pieces <- tolower(pieces)
 
-  is_value <- grepl("=", pieces)
+  is_value <- grepl("=", pieces, fixed = TRUE)
   flags <- pieces[!is_value]
 
   keyvalues <- strsplit(pieces[is_value], "\\s*=\\s*")
