@@ -100,9 +100,22 @@ new_response <- function(method,
   )
 }
 
+create_response <- function(req, curl_data, body) {
+  the$last_response <- new_response(
+    method = req_method_get(req),
+    url = curl_data$url,
+    status_code = curl_data$status_code,
+    headers = as_headers(curl_data$headers),
+    body = body,
+    request = req
+  )
+
+  the$last_response
+}
+
 
 #' @export
-print.httr2_response <- function(x,...) {
+print.httr2_response <- function(x, ...) {
   cli::cli_text("{.cls {class(x)}}")
   cli::cli_text("{.strong {x$method}} {x$url}")
   cli::cli_text("{.field Status}: {x$status_code} {resp_status_desc(x)}")
@@ -113,10 +126,12 @@ print.httr2_response <- function(x,...) {
   body <- x$body
   if (!resp_has_body(x)) {
     cli::cli_text("{.field Body}: None")
-  } else if (is_path(body)) {
-    cli::cli_text("{.field Body}: On disk {.path {body}} ({file.size(body)} bytes)")
   } else {
-    cli::cli_text("{.field Body}: In memory ({length(body)} bytes)")
+    switch(resp_body_type(x),
+      disk = cli::cli_text("{.field Body}: On disk {.path {body}} ({file.size(body)} bytes)"),
+      memory = cli::cli_text("{.field Body}: In memory ({length(body)} bytes)"),
+      stream = cli::cli_text("{.field Body}: Streaming connection")
+    )
   }
 
   invisible(x)
@@ -129,7 +144,7 @@ print.httr2_response <- function(x,...) {
 #' servers compress at least the body, and HTTP/2 can also compress the
 #' headers), but it conveys the same information.
 #'
-#' @param resp An HTTP [response]
+#' @inheritParams resp_headers
 #' @returns `resp` (invisibly).
 #' @export
 #' @examples
