@@ -52,6 +52,17 @@ test_that("can set query params", {
   expect_equal(req_url_query(req, !!!list(a = 1, a = 2))$url, "http://example.com/?a=1&a=2")
 })
 
+test_that("can control space handling", {
+  req <- request("http://example.com/")
+  expect_equal(req_url_query(req, a = " ")$url, "http://example.com/?a=%20")
+  expect_equal(req_url_query(req, a = " ", .space = "form")$url, "http://example.com/?a=+")
+
+  expect_snapshot(
+    req_url_query(req, a = " ", .space = "bar"),
+    error = TRUE
+  )
+})
+
 test_that("can handle multi query params", {
   req <- request("http://example.com/")
 
@@ -96,6 +107,12 @@ test_that("can opt-out of query escaping", {
   expect_equal(req_url_query(req, a = I(","))$url, "http://example.com/?a=,")
 })
 
+test_that("can construct relative urls", {
+  req <- request("http://example.com/a/b/c.html")
+  expect_equal(req_url_relative(req, ".")$url, "http://example.com/a/b/")
+  expect_equal(req_url_relative(req, "..")$url, "http://example.com/a/")
+  expect_equal(req_url_relative(req, "/d/e/f")$url, "http://example.com/d/e/f")
+})
 # explode -----------------------------------------------------------------
 
 test_that("explode handles expected inputs", {
