@@ -4,6 +4,8 @@ test_that("response has basic print method", {
   writeBin("sample content", "path-content")
 
   withr::defer(unlink(c("path-empty", "path-content")))
+  con <- file()
+  withr::defer(close(con))
 
   expect_snapshot({
     response(200)
@@ -11,6 +13,7 @@ test_that("response has basic print method", {
     response(200, body = charToRaw("abcdef"))
     response(200, body = new_path("path-empty"))
     response(200, body = new_path("path-content"))
+    response(200, body = con)
   })
 })
 
